//  URL   
// : NetSpirit

unit uri;

interface

uses Windows, Kol;

type

  TURIOptionalFields = (ofAuthInfo, ofHost, ofParams, ofBookmark);
  TURIOptionalFieldsSet = set of TURIOptionalFields;
  
  TURI = class
    protected
      FDocument: string;
      FProtocol: string;
      FURI: string;
      FPort: string;
      Fpath: string;
      FHost: string;
      FBookmark: string;
      FUserName: string;
      FPassword: string;
      FParams: string;

      procedure SetURI(const Value: string);
      function GetURI: string;
    public
      constructor Create(const AURI: string = '');
      function GetFullURI(const AOptionalFileds: TURIOptionalFieldsSet = [ofAuthInfo, ofHost, ofParams, ofBookmark]): string;
      
      property Bookmark : string read FBookmark write FBookMark;
      property Document: string read FDocument write FDocument;
      property Host: string read FHost write FHost;
      property Password: string read FPassword write FPassword;
      property Path: string read FPath write FPath;
      property Params: string read FParams write FParams;
      property Port: string read FPort write FPort;
      property Protocol: string read FProtocol write FProtocol;
      property URI: string read GetURI write SetURI;
      property Username: string read FUserName write FUserName;
  end;

   
implementation


function RPos(const ASub, AIn: string; AStart: Integer = -1): Integer;
var
  i: Integer;
  LStartPos: Integer;
  LTokenLen: Integer;
begin
  Result := 0;
  LTokenLen := Length(ASub);
  if AStart = -1 then
  begin
    AStart := Length(AIn);
  end;
  if AStart < (Length(AIn) - LTokenLen + 1) then
  begin
    LStartPos := AStart;
  end
  else
  begin
    LStartPos := (Length(AIn) - LTokenLen + 1);
  end;
  for i := LStartPos downto 1 do
  begin
    if StrComp(PChar(Copy(AIn, i, LTokenLen)), PChar(ASub)) = 0 then
    begin
      Result := i;
      Break;
    end;
  end;
end;

procedure NormalizePath(var APath: string);
var
  i: Integer;
begin
  // Normalize the directory delimiters to follow the UNIX syntax
  i := 1;
  while i <= Length(APath) do
  begin
    if APath[i] = '\' then APath[i] := '/';
    Inc(i, 1);
  end;
end;

constructor TURI.Create(const AURI: string = '');
begin
  inherited Create;
  if Length(AURI) > 0 then URI := AURI;
end;

procedure TURI.SetURI(const Value: string);
var
  LBuffer: string;
  LTokenPos, LParamsPos: Integer;
  LURI: string;
begin
  FURI := Value;
  NormalizePath(FURI);
  LURI := FURI;
  FHost := '';
  FProtocol := '';
  FPath := '';
  FDocument := '';
  FPort := '';
  FBookmark := '';
  FUsername := '';
  FPassword := '';
  FParams := '';

  LTokenPos := IndexOfStr(LURI, '://');

  if LTokenPos > 0 then
  begin
    // Get the protocol
    FProtocol := Copy(LURI, 1, LTokenPos  - 1);
    Delete(LURI, 1, LTokenPos + 2);
  end;
  
    // Get the user name, password, host and the port number
    
    LBuffer := Parse(LURI, '/');
    // Get username and password
    LTokenPos := IndexOfStr(LBuffer, '@');
    FPassword := Copy(LBuffer, 1, LTokenPos  - 1);
    if LTokenPos > 0 then
      Delete(LBuffer, 1, LTokenPos);
    FUserName := Parse(FPassword, ':');
    // Ignore cases where there is only password (http://:password@host/pat/doc)
    if Length(FUserName) = 0 then
    begin
      FPassword := '';
    end;
    
    // Get the host and the port number
    FHost := Parse(LBuffer, ':');
    FPort := LBuffer;
    
    // Get the path
    LParamsPos := IndexOfStr(LURI, '?');
    if LParamsPos > 0 then // The case when there is parameters after the document name '?'
    begin
      LTokenPos := RPos('/', LURI, LParamsPos);
    end
    else
    begin
      LParamsPos := IndexOfStr(LURI, '=');
      if LParamsPos > 0 then // The case when there is parameters after the document name '='
      begin
        LTokenPos := RPos('/', LURI, LParamsPos);
      end
      else
      begin
        LTokenPos := RPos('/', LURI, -1);
      end;
    end;

    FPath := '/' + Copy(LURI, 1, LTokenPos);
    // Get the document
    if LParamsPos > 0 then
    begin
      FDocument := Copy(LURI, 1, LParamsPos - 1);
      //Delete(LURI, 1, LParamsPos - 1);
      Delete(LURI, 1, LParamsPos); //   '?'
      FParams := LURI;
    end
    else FDocument := LURI;
    
    Delete(FDocument, 1, LTokenPos);

    FBookmark := FDocument;
    FDocument := Parse(FBookmark, '#');

  // Parse the # bookmark from the document
  if Length(FBookmark) = 0 then
  begin
    FBookmark := FParams;
    FParams := Parse(FBookmark, '#');
  end;
  
  FProtocol := LowerCase(FProtocol);
  FHost := LowerCase(FHost);
end;


function TURI.GetFullURI(const AOptionalFileds: TURIOptionalFieldsSet): string;
begin
  Result := '';
  if (ofHost in AOptionalFileds) and (Length(FHost) > 0) then // If Host specified, we can return Proto, User, Pass, Host and Port
  begin
    if Length(FProtocol) > 0 then
    begin
      Result := FProtocol + '://';
    end;

    if (ofAuthInfo in AOptionalFileds) and (Length(FUserName) > 0) then
    begin
      Result := Result + FUserName;

      if Length(FPassword) > 0 then
      begin
        Result := Result + ':' + FPassword;
      end;

      Result := Result + '@';
    end;

    Result := Result + FHost;
    
    if Length(FPort) > 0 then
    begin
      Result := Result + ':' + FPort;
    end;
  end;
  
  Result := Result + FPath + FDocument;
  if (ofParams in AOptionalFileds) and (Length(FParams) > 0) then
  begin
    Result := Result + '?' + FParams;
  end;
  
  if (ofBookmark in AOptionalFileds) and (Length(FBookmark) > 0) then
  begin
    Result := Result + '#' + FBookmark;
  end;
end;

function TURI.GetURI: string;
begin
  // result must contain only the proto://host/path/document
  // If you need the full URI then you have to call GetFullURI
  Result := GetFullURI([ofHost]);
end;


end.